import json
import uuid
from datetime import datetime

import boto3
import pytest

from nops_sink import S3Sink


@pytest.fixture
def sink():
    prefix = "/nopssinktest/output"
    bucket = "nikita-data"

    sink = S3Sink(
        bucket=bucket,
        prefix=prefix,
        session=boto3.Session(),
    )
    return sink


def test_s3sink_json_upload(sink):
    random_string = str(uuid.uuid4())
    filename = "testing_json"
    datetime_obj = datetime.now()
    payload = {"test": random_string, "date": datetime_obj}
    sink.put(payload=payload, name=filename)

    response = sink.get(name=filename)
    assert response["date"] == datetime_obj.isoformat()
    payload["date"] = datetime_obj.isoformat()
    assert response == payload

    sink.delete(name=filename)

    with pytest.raises(FileNotFoundError):
        assert sink.get(name=filename)


def test_s3sink_file_binary_upload(sink):
    filename = "testing_file"
    random_string = str(uuid.uuid4())
    with open("/tmp/testfile", "wb") as f:
        f.write(random_string.encode("utf-8"))

    with open("/tmp/testfile", "rb") as f:
        sink.put_file(payload=f, name=filename)

    assert sink.get_file(name=filename) == random_string.encode("utf-8")

    sink.delete(name=filename)



import io
import json
from datetime import date
from datetime import datetime
from typing import Any
from typing import Dict
from typing import Union

import boto3
import msgspec
import smart_open


def json_serial(obj):
    """
    JSON serializer for objects not serializable by default.
    """

    if isinstance(obj, (datetime, date)):
        return obj.isoformat()

    else:
        return str(obj)


class S3Sink:
    """
    Uploads data to the bucket. In case of JSON/str data it is automatically compressed
    to gz and output name is changed to reflect that.
    """

    def __init__(self, session: boto3.Session, bucket: str, prefix: str):
        self.session = session
        self.bucket = bucket
        if not prefix.endswith("/"):
            prefix = f"{prefix}/"

        if prefix.startswith("/"):
            prefix = prefix[1:]

        self.prefix = prefix

        self._client = self.session.client("s3")

    def _s3_path(self, name: str, is_file=False):
        if is_file:
            return f"s3://{self.bucket}/{self.prefix}{name}"
        else:
            return f"s3://{self.bucket}/{self.prefix}{name}.json.gz"

    def put_file(
        self,
        payload: Union[io.BufferedReader, io.TextIOWrapper],
        name: str,
        buffer_size=1024,
        encoding="utf-8",
    ) -> bool:
        open_kwargs: Dict[str, Any] = dict(
            uri=self._s3_path(name=name, is_file=True), mode="wb", transport_params=dict(client=self._client)
        )

        with smart_open.open(**open_kwargs) as f:
            if isinstance(payload, io.BufferedReader):
                data = payload.read(buffer_size)
                while data:
                    f.write(data)
                    data = payload.read(buffer_size)

            elif isinstance(payload, io.TextIOWrapper):
                data = payload.read(buffer_size).encode(encoding=encoding)
                while data:
                    f.write(data)
                    data = payload.read(buffer_size)

            else:
                raise ValueError(f"{type(payload)} payload type is not supported")
        return True

    def put(self, payload: Union[str, dict], name: str) -> bool:
        open_kwargs: Dict[str, Any] = dict(
            uri=self._s3_path(name=name), mode="wb", transport_params=dict(client=self._client), compression=".gz"
        )

        with smart_open.open(**open_kwargs) as f:
            if isinstance(payload, str):
                f.write(payload.encode(encoding="UTF-8"))

            elif isinstance(payload, dict):
                data = msgspec.json.encode(payload, enc_hook=json_serial)
                f.write(data)

            else:
                raise ValueError(f"{type(payload)} payload type is not supported")

        return True

    def get(self, name: str):
        try:
            open_kwargs = dict(
                uri=self._s3_path(name=name), compression=".gz", transport_params=dict(client=self._client), mode="rb"
            )

            with smart_open.open(**open_kwargs) as f:
                response = f.read()
                return msgspec.json.decode(response)
                return response

        except OSError as e:
            raise FileNotFoundError(e)

    def get_file(self, name: str):
        try:
            open_kwargs = dict(
                uri=self._s3_path(name=name, is_file=True), transport_params=dict(client=self._client), mode="rb"
            )

            with smart_open.open(**open_kwargs) as f:
                return f.read()

        except OSError as e:
            raise FileNotFoundError(e)

    def delete_file(self, name: str) -> bool:
        self.session.client("s3").delete_object(Bucket=self.bucket, Key=f"{self.prefix}{name}")
        return True

    def delete(self, name: str) -> bool:
        self.session.client("s3").delete_object(Bucket=self.bucket, Key=f"{self.prefix}{name}.json.gz")
        return True



from functools import lru_cache

from nops_kafka import Producer
from nops_kafka import ensure_topics


class KafkaSink:
    """
    Usage:

            message_sink = KafkaSink(
                topic="data.output",
                headers={"client_id": "1337"},
                bootstrap_servers=settings.KAFKA_URL,
            )
            message_sink.put(payload={"testing": True})
            message_sink.flush()

    """

    def __init__(self, topic, headers, bootstrap_servers):
        self.bootstrap_servers = bootstrap_servers
        self.producer = Producer(bootstrap_servers=bootstrap_servers)
        self.topic = topic
        self.headers = headers

    @lru_cache
    def _ensure_topics(self) -> bool:
        return ensure_topics(bootstrap_servers=self.bootstrap_servers, required_topics=[self.topic])

    def put(self, payload):
        self._ensure_topics()
        self.producer.send(self.topic, value=payload, headers=self.headers)

    def flush(self) -> bool:
        self.producer.flush()
        return True



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nops_sink', 'nops_sink.tests']

package_data = \
{'': ['*']}

install_requires = \
['boto3>1.22', 'msgspec>0.7.0', 'nops_kafka>0.1', 'smart_open>4.0']

setup_kwargs = {
    'name': 'nops-sink',
    'version': '0.3.1',
    'description': 'Set of modules to simplify submission of data to the various sinks.',
    'long_description': None,
    'author': 'nOps Engineers',
    'author_email': 'eng@nops.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
}


setup(**setup_kwargs)
